import json
from operator import mul, truediv
from .formatters import Numeric


class Salary:
    """ Class for calculation and conversion of salary by time period """

    _period_yearly_defaults = {
        'hour': 2080,
        'day': 260,
        'week': 52,
        'fortnight': 26,
        'month': 12,
        'quarter': 4,
        'semester': 2,
        'year': 1,
    }

    def __init__(self, *args, **kwargs):
        """ Salary initialization

        Arguments:
            args[0]: required: salary amount (float)
            args[1]: required: salary amount period (string)
                     valid options: [hour|day|week|fortnight|month|quarter|semester|year]

        Keyword arguments:
            kwargs['hours']: custom hours in year (int) (default: 2080)
            kwargs['days']: custom days in year (int) (default: 260)
            kwargs['weeks']: custom weeks in year (int) (default: 52)
            kwargs['fortnights']: custom fortnights in year (int) (default: 26)
            kwargs['months']: custom months in year (int) (default: 12)
            kwargs['quarters']: custom quarters in year (int) (default: 4)
            kwargs['semesters']: custom semesters in year (int) (default: 2)

        Examples:
            Salary(15, 'hour')
            Salary(31200, 'year')
            Salary(15, 'hour', hours=1040, days=130, weeks=26)
        """
        self._init_yearly_occurrences()
        self._handle_args(args)
        self._handle_kwargs(kwargs)

    def __repr__(self):
        out_str = 'Salary('
        out_str += f"{self.amount}, '{self.period}'"
        for key in self._period_yearly_defaults:
            value = getattr(self, f'{key}s_in_year')
            if value != self._period_yearly_defaults[key]:
                out_str += f', {key}s={value}'
        out_str += ')'
        return out_str

    def __str__(self):
        return f'{self.amount.dollars} per {self.period}'

    def _init_yearly_occurrences(self):
        self.hours_in_year = self._period_yearly_defaults['hour']
        self.days_in_year = self._period_yearly_defaults['day']
        self.weeks_in_year = self._period_yearly_defaults['week']
        self.fortnights_in_year = self._period_yearly_defaults['fortnight']
        self.months_in_year = self._period_yearly_defaults['month']
        self.quarters_in_year = self._period_yearly_defaults['quarter']
        self.semesters_in_year = self._period_yearly_defaults['semester']
        self.years_in_year = self._period_yearly_defaults['year']

    def _handle_args(self, args):
        try:
            self.amount = Numeric(args[0])
        except (IndexError, ValueError) as err:
            raise err

        try:
            if args[1] in self._period_yearly_defaults:
                self.period = args[1]
            else:
                raise ValueError(f'Invalid argument provided: {args[1]}')
        except (IndexError, ValueError) as err:
            raise err

    def _handle_kwargs(self, kwargs):
        for k, v in kwargs.items():
            if k[:-1] in self._period_yearly_defaults and k[:-1] != 'year':
                setattr(self, f'{k}_in_year', int(v))

    def per_period(self, amount, period, operation=truediv):
        """ Calculate amount per given period using operation callback function

        Parameters:
            amount: required: the amount to base calculation on (Decimal)
            period: required: the period to use for the calculation (str)
            operation: optional: which operator to use against (amount, period) (function)
        """
        return Numeric(operation(amount, getattr(self, f'{period}s_in_year')))

    def serialize_per_period(self, format='float'):
        obj = {}
        for period in self._period_yearly_defaults:
            obj[period] = getattr(self.per_period(self.yearly.decimal, period), format)
        return json.dumps(obj)

    def serialize_times_per_year(self):
        obj = {}
        for period in self._period_yearly_defaults:
            obj[period] = getattr(self, f'{period}s_in_year')
        return json.dumps(obj)

    def serialize(self):
        obj = {}
        obj['amount'] = self.amount.float
        obj['period'] = self.period
        obj['per_period_numeric'] = json.loads(self.serialize_per_period(format='float'))
        obj['per_period_dollars'] = json.loads(self.serialize_per_period(format='dollars'))
        obj['times_per_year'] = json.loads(self.serialize_times_per_year())
        return json.dumps(obj)

    @property
    def yearly(self):
        return self.per_period(self.amount.decimal, self.period, operation=mul)

    @property
    def hourly(self):
        return self.per_period(self.yearly.decimal, 'hour')

    @property
    def daily(self):
        return self.per_period(self.yearly.decimal, 'day')

    @property
    def weekly(self):
        return self.per_period(self.yearly.decimal, 'week')

    @property
    def fortnightly(self):
        return self.per_period(self.yearly.decimal, 'fortnight')

    @property
    def monthly(self):
        return self.per_period(self.yearly.decimal, 'month')

    @property
    def quarterly(self):
        return self.per_period(self.yearly.decimal, 'quarter')

    @property
    def semesterly(self):
        return self.per_period(self.yearly.decimal, 'semester')



from decimal import Decimal, InvalidOperation


class Numeric:
    """ Object to store and convert a numeric value to various other formats
    """

    def __init__(self, value):
        if isinstance(value, Numeric):
            self.value = Decimal(value.decimal)
        else:
            try:
                self.value = Decimal(value)
            except (InvalidOperation):
                raise ValueError(f'Value not numeric: {value}')

    def __repr__(self):
        return f'Numeric({repr(self.decimal)})'

    def __str__(self):
        return f'{self.decimal}'

    @staticmethod
    def format_dollars(value):
        """ Convert number value to Dollars, e.g.: 12345 -> "$12,345.00" """
        return f'${value:,.2f}'

    @property
    def dollars(self):
        return self.format_dollars(self.value)

    @property
    def float(self):
        return float(self.value)

    @property
    def int(self):
        return int(self.value)

    @property
    def decimal(self):
        return Decimal(self.value)



import json
import unittest
from decimal import Decimal
from wage import Salary


class TestSalary(unittest.TestCase):

    def test_invalid_input_invalid_amount(self):
        with self.assertRaises(ValueError):
            Salary('15a')

    def test_invalid_input_no_period(self):
        with self.assertRaises(IndexError):
            Salary(15)

    def test_invalid_input_invalid_period(self):
        with self.assertRaises(ValueError):
            Salary(15, 'invalid')

    def test_invalid_input_invalid_kwarg_key(self):
        s = Salary(15, 'hour', centuries=1)
        with self.assertRaises(AttributeError):
            s.centuries_in_year

    def test_invalid_input_invalid_kwarg_value(self):
        with self.assertRaises(ValueError):
            Salary(15, 'hour', hours='1a')

    def test_valid_input_minimal(self):
        s1 = Salary(15, 'hour')
        s2 = Salary(15.0, 'hour')
        s3 = Salary('15', 'hour')

        self.assertIsNotNone(s1)
        self.assertIsNotNone(s2)
        self.assertIsNotNone(s3)

        self.assertIsNotNone(s1.amount.decimal)
        self.assertIsNotNone(s2.amount.decimal)
        self.assertIsNotNone(s3.amount.decimal)

        self.assertIsNotNone(s1.period)
        self.assertIsNotNone(s2.period)
        self.assertIsNotNone(s3.period)

        self.assertIsInstance(s1, Salary)
        self.assertIsInstance(s2, Salary)
        self.assertIsInstance(s3, Salary)

        self.assertIsInstance(s1.amount.decimal, Decimal)
        self.assertIsInstance(s2.amount.decimal, Decimal)
        self.assertIsInstance(s3.amount.decimal, Decimal)

        self.assertIsInstance(s1.period, str)
        self.assertIsInstance(s2.period, str)
        self.assertIsInstance(s3.period, str)

        self.assertEqual(s1.amount.decimal, 15.0)
        self.assertEqual(s2.amount.decimal, 15.0)
        self.assertEqual(s3.amount.decimal, 15.0)

        self.assertEqual(s1.period, 'hour')
        self.assertEqual(s2.period, 'hour')
        self.assertEqual(s3.period, 'hour')

    def test_default_periods_in_year(self):
        s = Salary(15, 'hour')
        self.assertEqual(s.hours_in_year, 2080)
        self.assertEqual(s.days_in_year, 260)
        self.assertEqual(s.weeks_in_year, 52)
        self.assertEqual(s.fortnights_in_year, 26)
        self.assertEqual(s.months_in_year, 12)
        self.assertEqual(s.quarters_in_year, 4)
        self.assertEqual(s.semesters_in_year, 2)
        self.assertEqual(s.years_in_year, 1)

    def test_custom_periods_in_year_year_ineffective(self):
        s = Salary(15, 'hour', years=3)
        self.assertEqual(s.years_in_year, 1)

    def test_custom_period_value_int_casting(self):
        s = Salary(15, 'hour', hours=2075.5)
        self.assertEqual(s.hours_in_year, 2075)

    def test_custom_periods_in_year_full(self):
        s = Salary(15, 'hour',
                   hours=1040,
                   days=130,
                   weeks=26,
                   fortnights=13,
                   months=10,
                   quarters=2,
                   semesters=1)
        self.assertEqual(s.hours_in_year, 1040)
        self.assertEqual(s.days_in_year, 130)
        self.assertEqual(s.weeks_in_year, 26)
        self.assertEqual(s.fortnights_in_year, 13)
        self.assertEqual(s.months_in_year, 10)
        self.assertEqual(s.quarters_in_year, 2)
        self.assertEqual(s.semesters_in_year, 1)

    def test_custom_periods_in_year_partial(self):
        s1 = Salary(15, 'hour', hours=1040, days=130)
        self.assertEqual(s1.hours_in_year, 1040)
        self.assertEqual(s1.days_in_year, 130)
        self.assertEqual(s1.weeks_in_year, 52)
        self.assertEqual(s1.fortnights_in_year, 26)
        self.assertEqual(s1.months_in_year, 12)
        self.assertEqual(s1.quarters_in_year, 4)
        self.assertEqual(s1.semesters_in_year, 2)

        s2 = Salary(15600, 'year', hours=1040, days=130)
        self.assertEqual(s2.hours_in_year, 1040)
        self.assertEqual(s2.days_in_year, 130)
        self.assertEqual(s2.weeks_in_year, 52)
        self.assertEqual(s2.fortnights_in_year, 26)
        self.assertEqual(s2.months_in_year, 12)
        self.assertEqual(s2.quarters_in_year, 4)
        self.assertEqual(s2.semesters_in_year, 2)

    def test_salary_period_calculations(self):
        s1 = Salary(15, 'hour')
        self.assertEqual(s1.hourly.decimal, Decimal(15))
        self.assertEqual(s1.daily.decimal, Decimal(120))
        self.assertEqual(s1.weekly.decimal, Decimal(600))
        self.assertEqual(s1.fortnightly.decimal, Decimal(1200))
        self.assertEqual(s1.monthly.decimal, Decimal(2600))
        self.assertEqual(s1.quarterly.decimal, Decimal(7800))
        self.assertEqual(s1.semesterly.decimal, Decimal(15600))
        self.assertEqual(s1.yearly.decimal, Decimal(31200))

        s2 = Salary(15, 'hour',
                    hours=1040,
                    days=130,
                    weeks=26,
                    fortnights=13,
                    months=10,
                    quarters=2,
                    semesters=1)
        self.assertEqual(s2.hourly.decimal, Decimal(15))
        self.assertEqual(s2.daily.decimal, Decimal(120))
        self.assertEqual(s2.weekly.decimal, Decimal(600))
        self.assertEqual(s2.fortnightly.decimal, Decimal(1200))
        self.assertEqual(s2.monthly.decimal, Decimal(1560))
        self.assertEqual(s2.quarterly.decimal, Decimal(7800))
        self.assertEqual(s2.semesterly.decimal, Decimal(15600))
        self.assertEqual(s2.yearly.decimal, Decimal(15600))

        s3 = Salary(15, 'hour', hours=1040, days=130)
        self.assertEqual(s3.hourly.decimal, Decimal(15))
        self.assertEqual(s3.daily.decimal, Decimal(120))
        self.assertEqual(s3.weekly.decimal, Decimal(300))
        self.assertEqual(s3.fortnightly.decimal, Decimal(600))
        self.assertEqual(s3.monthly.decimal, Decimal(1300))
        self.assertEqual(s3.quarterly.decimal, Decimal(3900))
        self.assertEqual(s3.semesterly.decimal, Decimal(7800))
        self.assertEqual(s3.yearly.decimal, Decimal(15600))

        s4 = Salary(15600, 'year', hours=1040, days=130)
        self.assertEqual(s4.hourly.decimal, Decimal(15))
        self.assertEqual(s4.daily.decimal, Decimal(120))
        self.assertEqual(s4.weekly.decimal, Decimal(300))
        self.assertEqual(s4.fortnightly.decimal, Decimal(600))
        self.assertEqual(s4.monthly.decimal, Decimal(1300))
        self.assertEqual(s4.quarterly.decimal, Decimal(3900))
        self.assertEqual(s4.semesterly.decimal, Decimal(7800))
        self.assertEqual(s4.yearly.decimal, Decimal(15600))

    def test_serialize_per_period_float(self):
        s = Salary(31200, 'year')
        per_period_float = json.loads(s.serialize_per_period(format='float'))
        self.assertIsNotNone(per_period_float)
        self.assertIsInstance(per_period_float, dict)
        self.assertIsNotNone(per_period_float['hour'])
        self.assertIsNotNone(per_period_float['day'])
        self.assertIsNotNone(per_period_float['week'])
        self.assertIsNotNone(per_period_float['fortnight'])
        self.assertIsNotNone(per_period_float['month'])
        self.assertIsNotNone(per_period_float['quarter'])
        self.assertIsNotNone(per_period_float['semester'])
        self.assertIsNotNone(per_period_float['year'])
        self.assertEqual(per_period_float['hour'], 15.0)
        self.assertEqual(per_period_float['day'], 120.0)
        self.assertEqual(per_period_float['week'], 600.0)
        self.assertEqual(per_period_float['fortnight'], 1200.0)
        self.assertEqual(per_period_float['month'], 2600.0)
        self.assertEqual(per_period_float['quarter'], 7800.0)
        self.assertEqual(per_period_float['semester'], 15600.0)
        self.assertEqual(per_period_float['year'], 31200.0)

    def test_serialize_per_period_dollars(self):
        s = Salary(31200, 'year')
        per_period_dollars = json.loads(s.serialize_per_period(format='dollars'))
        self.assertIsNotNone(per_period_dollars)
        self.assertIsInstance(per_period_dollars, dict)
        self.assertIsNotNone(per_period_dollars['hour'])
        self.assertIsNotNone(per_period_dollars['day'])
        self.assertIsNotNone(per_period_dollars['week'])
        self.assertIsNotNone(per_period_dollars['fortnight'])
        self.assertIsNotNone(per_period_dollars['month'])
        self.assertIsNotNone(per_period_dollars['quarter'])
        self.assertIsNotNone(per_period_dollars['semester'])
        self.assertIsNotNone(per_period_dollars['year'])
        self.assertEqual(per_period_dollars['hour'], '$15.00')
        self.assertEqual(per_period_dollars['day'], '$120.00')
        self.assertEqual(per_period_dollars['week'], '$600.00')
        self.assertEqual(per_period_dollars['fortnight'], '$1,200.00')
        self.assertEqual(per_period_dollars['month'], '$2,600.00')
        self.assertEqual(per_period_dollars['quarter'], '$7,800.00')
        self.assertEqual(per_period_dollars['semester'], '$15,600.00')
        self.assertEqual(per_period_dollars['year'], '$31,200.00')

    def test_serialize_times_per_year(self):
        s = Salary(31200, 'year')
        times_per_year = json.loads(s.serialize_times_per_year())
        self.assertIsNotNone(times_per_year)
        self.assertIsInstance(times_per_year, dict)
        self.assertIsNotNone(times_per_year['hour'])
        self.assertIsNotNone(times_per_year['day'])
        self.assertIsNotNone(times_per_year['week'])
        self.assertIsNotNone(times_per_year['fortnight'])
        self.assertIsNotNone(times_per_year['month'])
        self.assertIsNotNone(times_per_year['quarter'])
        self.assertIsNotNone(times_per_year['semester'])
        self.assertIsNotNone(times_per_year['year'])
        self.assertEqual(times_per_year['hour'], 2080)
        self.assertEqual(times_per_year['day'], 260)
        self.assertEqual(times_per_year['week'], 52)
        self.assertEqual(times_per_year['fortnight'], 26)
        self.assertEqual(times_per_year['month'], 12)
        self.assertEqual(times_per_year['quarter'], 4)
        self.assertEqual(times_per_year['semester'], 2)
        self.assertEqual(times_per_year['year'], 1)

    def test_serialize(self):
        s = Salary(31200, 'year')
        obj = json.loads(s.serialize())
        self.assertIsNotNone(obj)
        self.assertIsInstance(obj, dict)
        self.assertIsNotNone(obj['amount'])
        self.assertIsNotNone(obj['period'])
        self.assertIsNotNone(obj['per_period_numeric'])
        self.assertIsNotNone(obj['per_period_dollars'])
        self.assertIsNotNone(obj['times_per_year'])
        self.assertIsInstance(obj['per_period_numeric'], dict)
        self.assertIsInstance(obj['per_period_dollars'], dict)
        self.assertIsInstance(obj['times_per_year'], dict)
        self.assertEqual(obj['amount'], 31200.0)
        self.assertEqual(obj['period'], 'year')


if __name__ == '__main__':
    unittest.main()



import unittest
from decimal import Decimal
from wage import Numeric


class TestFormatters(unittest.TestCase):

    def test_numeric_invalid_input(self):
        with self.assertRaises(ValueError):
            Numeric('1a')

    def test_numeric_valid_input(self):
        n = Numeric(1)
        self.assertIsNotNone(n)
        self.assertIsInstance(n, Numeric)
        self.assertIsNotNone(n.value)
        self.assertIsInstance(n.value, Decimal)
        self.assertEqual(n.value, 1.0)

    def test_numeric_format_dollars(self):
        value = Decimal(1234.5612)
        dollars = Numeric.format_dollars(value)
        self.assertEqual(dollars, '$1,234.56')

    def test_numeric_dollars(self):
        n1 = Numeric(1.5)
        n2 = Numeric(300)
        n3 = Numeric(Decimal(1234.5612))
        self.assertIsNotNone(n1)
        self.assertIsNotNone(n2)
        self.assertIsNotNone(n3)
        self.assertIsInstance(n1.dollars, str)
        self.assertIsInstance(n2.dollars, str)
        self.assertIsInstance(n3.dollars, str)
        self.assertEqual(n1.dollars, '$1.50')
        self.assertEqual(n2.dollars, '$300.00')
        self.assertEqual(n3.dollars, '$1,234.56')

    def test_numeric_float(self):
        n1 = Numeric(1)
        n2 = Numeric(1234.5612)
        self.assertIsNotNone(n1)
        self.assertIsNotNone(n2)
        self.assertIsInstance(n1.float, float)
        self.assertIsInstance(n2.float, float)
        self.assertEqual(n1.float, 1.0)
        self.assertEqual(n2.float, 1234.5612)

    def test_numeric_int(self):
        n1 = Numeric(1)
        n2 = Numeric(1234.5612)
        self.assertIsNotNone(n1)
        self.assertIsNotNone(n2)
        self.assertIsInstance(n1.int, int)
        self.assertIsInstance(n2.int, int)
        self.assertEqual(n1.int, 1)
        self.assertEqual(n2.int, 1234)

    def test_numeric_decimal(self):
        n1 = Numeric(1)
        n2 = Numeric(1234.5612)
        self.assertIsNotNone(n1)
        self.assertIsNotNone(n2)
        self.assertIsInstance(n1.decimal, Decimal)
        self.assertIsInstance(n2.decimal, Decimal)
        self.assertEqual(n1.decimal, Decimal(1.0))
        self.assertEqual(n2.decimal, Decimal(1234.5612))


if __name__ == '__main__':
    unittest.main()



import sys
from .salary import Salary


def salary_report(salary):
    periods = [
        'hourly',
        'daily',
        'weekly',
        'fortnightly',
        'monthly',
        'quarterly',
        'semesterly',
        'yearly'
    ]

    out_str = ""
    for period in periods:
        amount = getattr(salary, period)
        out_str += f'{period.title():15}: {amount.dollars}\n'
    return out_str


def main(args=sys.argv[1:]):
    salary_args = []
    salary_kwargs = {}
    for arg in args:
        if '=' not in arg:
            salary_args.append(arg)
        else:
            k = arg.split('=')[0]
            v = arg.split('=')[1]
            salary_kwargs[k] = v
    salary = Salary(*salary_args, **salary_kwargs)
    print(salary_report(salary))


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages
from pathlib import Path
BASE_DIR = Path(__file__).parent
long_description = (BASE_DIR / "README.md").read_text()
setup(
    name="wage",
    version="0.5",
    packages=find_packages(),
    python_requires='>=3.7',
    install_requires=[],
    author='Marcus Bowman',
    author_email='miliarch.mb@gmail.com',
    description='A python module for modeling and converting salary/income information',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    keywords='wage salary income convert model',
    url='https://github.com/miliarch/wage',
    project_urls={
        'Source Code': 'https://github.com/miliarch/wage',
    },
    entry_points={
        'console_scripts': ['wage=wage.interface:main']
    }
)
